/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.persistence.db;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.logging.Logger;

import org.hibernate.FlushMode;
import org.hibernate.Query;
import org.hibernate.Session;
import org.jbpm.pvm.internal.hibernate.HibernatePvmDbSession;
import org.jbpm.pvm.internal.job.TimerImpl;
import org.jbpm.pvm.job.Timer;
import org.ow2.bonita.definition.GlobalClassData;
import org.ow2.bonita.definition.PackageClassData;
import org.ow2.bonita.definition.PackageDependency;
import org.ow2.bonita.definition.XpdlProcess;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.persistence.RuntimeDbSession;
import org.ow2.bonita.runtime.ActivityInstanceKey;
import org.ow2.bonita.runtime.PackageVersion;
import org.ow2.bonita.runtime.ProcessInstanceKey;
import org.ow2.bonita.runtime.ProcessVersion;
import org.ow2.bonita.runtime.TaskRunTime;
import org.ow2.bonita.runtime.XpdlExecution;
import org.ow2.bonita.runtime.XpdlInstance;


public class RuntimeDbSessionImpl extends HibernatePvmDbSession implements RuntimeDbSession {
  
  private static final Logger LOG = Logger.getLogger(RuntimeDbSessionImpl.class.getName());

  public RuntimeDbSessionImpl(Session session) {
    setSession(session);
    session.setFlushMode(FlushMode.COMMIT);
  }
  
  public TaskRunTime getTask(TaskUUID taskUUID) {
    Query query = getSession().getNamedQuery("getTask");
    query.setString("taskUUID", taskUUID.toString());
    query.setMaxResults(1);
    query.setCacheable(true);
    TaskRunTime task = (TaskRunTime) query.uniqueResult();
    return task;
  }
  
  @SuppressWarnings("unchecked")
  public Set<TaskRunTime> getInstanceTasks(ProcessInstanceUUID instanceUUID) {
    Set<TaskRunTime> result = new HashSet<TaskRunTime>();
    Query query = getSession().getNamedQuery("getInstanceTasks");
    query.setCacheable(true);
    query.setString("instanceUUID", instanceUUID.toString());
    result.addAll(query.list());
    return result;
  }
  
  public XpdlInstance getXpdlInstance(ProcessInstanceUUID instanceUUID) {
    Query query = getSession().getNamedQuery("getXpdlInstance");
    query.setString("instanceUUID", instanceUUID.toString());
    query.setCacheable(true);
    query.setMaxResults(1);
    return (XpdlInstance) query.uniqueResult();
  }

  @SuppressWarnings("unchecked")
  public Set<XpdlInstance> getXpdlInstances(ProcessDefinitionUUID processUUID) {
    Query query = getSession().getNamedQuery("getXpdlInstances");
    query.setCacheable(true);
    query.setString("processUUID", processUUID.toString());
    List<XpdlInstance> list = query.list();
    Set<XpdlInstance> result = new HashSet<XpdlInstance>();
    if (list != null && !list.isEmpty()) {
      result.addAll(list);
    }
    return result;
  }

  public XpdlProcess getXpdlProcess(ProcessDefinitionUUID processUUID) {
    Query query = getSession().getNamedQuery("getXpdlProcess");
    query.setString("processUUID", processUUID.toString());
    query.setCacheable(true);
    query.setMaxResults(1);
    XpdlProcess processDefinition = (XpdlProcess) query.uniqueResult();
    return processDefinition;
  }

  public PackageClassData getPackageClassData(PackageDefinitionUUID packageUUID) {
    Query query = getSession().getNamedQuery("getPackageClassData");
    query.setCacheable(true);
    query.setString("packageUUID", packageUUID.toString());
    query.setMaxResults(1);
    PackageClassData classData = (PackageClassData) query.uniqueResult();
    return classData;
  }
  
  public ProcessVersion getLastProcessVersion(String processId, String packageId) {
    Query query = getSession().getNamedQuery("getLastProcessVersion");
    query.setCacheable(true);
    query.setString("processId", processId);
    query.setString("packageId", packageId);
    query.setMaxResults(1);
    ProcessVersion pv = (ProcessVersion) query.uniqueResult();
    return pv;
  }
  
  public PackageVersion getLastPackageVersion(String packageId) {
    Query query = getSession().getNamedQuery("getLastPackageVersion");
    query.setCacheable(true);
    query.setString("packageId", packageId);
    query.setMaxResults(1);
    PackageVersion pv = (PackageVersion) query.uniqueResult();
    return pv;
  }

  public GlobalClassData getGlobalClassData(String className) {
    Query query = getSession().getNamedQuery("getGlobalClassData");
    query.setCacheable(true);
    query.setString("className", className);
    query.setMaxResults(1);
    GlobalClassData classData = (GlobalClassData) query.uniqueResult();
    return classData;
  }
  
  public XpdlExecution getExecutionPointingOnNode(
      ProcessInstanceUUID instanceUUID, String nodeName, String iterationId, String activityInstanceId) {
    Query query = getSession().getNamedQuery("findInstanceExecutionPointingOnNode");
    query.setCacheable(true);
    query.setString("instanceUUID", instanceUUID.toString());
    query.setString("nodeName", nodeName);
    query.setString("iterationId", iterationId);
    query.setString("activityInstanceId", activityInstanceId);
    query.setMaxResults(1);
    return (XpdlExecution) query.uniqueResult();
  }

  @SuppressWarnings("unchecked")
  public Set<PackageDependency> getPackageDependencies(String className) {
    final Query query = getSession().getNamedQuery("getPackageDependencies");
    query.setCacheable(true);
    query.setString("className", className);
    final List<PackageDependency> results = query.list();
    if (results != null) {
      return new HashSet<PackageDependency>(results);
    }
    return null;
  }

  public PackageDependency getPackageDependency(PackageDefinitionUUID packageUUID) {
    final Query query = getSession().getNamedQuery("getPackageDependency");
    query.setCacheable(true);
    query.setString("packageUUID", packageUUID.toString());
    return (PackageDependency) query.uniqueResult();
  }
  
  public XpdlProcess findLatestProcessById(String processId) {
    Query query = getSession().getNamedQuery("findLatestProcessById");
    query.setCacheable(true);
    query.setString("processId", processId);
    return (XpdlProcess)query.uniqueResult();
  }

  public ActivityInstanceKey getActivityInstanceKey(ProcessInstanceUUID instanceUUID,
      String activityName) {
    Query query = getSession().getNamedQuery("getActivityInstanceKey");
    query.setCacheable(true);
    query.setString("instanceUUID", instanceUUID.toString());
    query.setString("activityId", activityName);
    return (ActivityInstanceKey)query.uniqueResult();
  }

  public ProcessInstanceKey getProcessInstanceKey(ProcessDefinitionUUID processUUID) {
    Query query = getSession().getNamedQuery("getProcessInstanceKey");
    query.setCacheable(true);
    query.setString("processUUID", processUUID.toString());
    return (ProcessInstanceKey)query.uniqueResult();
  }

  @SuppressWarnings("unchecked")
  public Collection<Timer> getExecutionTimers(XpdlExecution xpdlExecution) {
    Query query = getSession().createQuery("from " + TimerImpl.class.getName() + " as timer where timer.execution = :exec");
    query.setEntity("exec", xpdlExecution);
    return query.list();
  }
  
  @SuppressWarnings("unchecked")
  public Set<Timer> getInstanceTimers(ProcessInstanceUUID instanceUUID) {
    Query query = getSession().getNamedQuery("getInstanceTimers");
    query.setCacheable(true);
    query.setString("instanceUUID", instanceUUID.toString());
    List<Timer> list = query.list();
    Set<Timer> result = new HashSet<Timer>();
    if (list != null && !list.isEmpty()) {
      result.addAll(list);
    }
    return result;
  }

}


